@@ -11,6 +11,7 @@ from .errors import (
     AutoDiscoverFailed,
     ErrorInternalServerError,
     ErrorNonExistentMailbox,
+    ErrorOrganizationNotFederated,
     ErrorServerBusy,
     InvalidTypeError,
 )
@@ -2077,12 +2078,6 @@ class UserResponse(EWSElement):
                 return version
         raise ValueError(f"Unknown supported schemas: {supported_schemas}")
 
-    @staticmethod
-    def _is_url(s):
-        if not s:
-            return False
-        return s.startswith("http://") or s.startswith("https://")
-
     def raise_errors(self):
         if self.error_code == "InvalidUser":
             raise ErrorNonExistentMailbox(self.error_message)
@@ -2107,6 +2102,8 @@ class UserResponse(EWSElement):
             raise ErrorInternalServerError(error_message)
         if error_code == "ServerBusy":
             raise ErrorServerBusy(error_message)
+        if error_code == "NotFederated":
+            raise ErrorOrganizationNotFederated(error_message)
         if error_code not in ("NoError", "RedirectAddress", "RedirectUrl"):
             return cls(error_code=error_code, error_message=error_message)
 
